@@ -1,5 +1,7 @@
+
+//OpenSCADA system file: main.cpp
 /***************************************************************************
- *   Copyright (C) 2004 by Roman Savochenko                                *
+ *   Copyright (C) 2003-2006 by Roman Savochenko                           *
  *   rom_as@fromru.com                                                     *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -20,38 +22,50 @@
 
 #include <locale.h>
 #include <unistd.h>
+#include <getopt.h>
 
 #include "terror.h"
-#include "tmessage.h"
+#include "tmess.h"
 #include "tsys.h"
-#include "tkernel.h"
 
 int main(int argc, char *argv[], char *envp[] )
 {
     int rez = 0, i_krn = 0;
-
-    //while(*envp) printf("%s\n",*envp++);
-    Mess = new TMessage();    
-    SYS  = new TSYS(argc,argv,envp);
-
-    Mess->checkCommandLine();
-    Mess->updateOpt();		
     
-    try
-    { 
-	while(true)
+    //Check demon mode start
+    int next_opt;
+    optind=opterr=0;
+    struct option long_opt[] = { {"demon" ,0,NULL,'d'}, {NULL    ,0,NULL,0  } };	
+    while((next_opt=getopt_long(argc,argv,"",long_opt,NULL)) != -1)
+	if( next_opt == 'd' )
 	{
-	    string k_name = SYS->cfgNode()->childGet("kernel",i_krn++)->attr("id");
-	    SYS->kern_add( k_name );
-	    if( !SYS->kern_at( k_name ).at().run() ) rez++;	    
+	    printf("Start into demon mode!\n");
+	    int pid = fork();
+	    if( pid == -1 )
+	    {	
+		printf("Error: fork error!\n");
+		return(-1);
+	    }
+	    if( pid != 0 )	return(0);
+	    
+	    //Prepare demon environment
+	    setsid();
+	    
+	    break;	    
 	}
-    }
-    catch(...) { }    
-    if( rez ) SYS->start();        
     
-    delete SYS;
-    delete Mess;    
+    //while(*envp) printf("%s\n",*envp++);
+    try
+    {
+	SYS = new TSYS(argc,argv,envp);	
+	
+	SYS->load();
+	rez = SYS->start();    
+	
+	delete SYS;
+    }catch(TError err)
+    { Mess->put(err.cat.c_str(),TMess::Error,"%s",err.mess.c_str()); }
 
-    return(rez);
+    return rez;
 }
 
