@@ -1,5 +1,7 @@
- /***************************************************************************
- *   Copyright (C) 2004 by Roman Savochenko                                *
+
+//OpenSCADA system file: tconfig.h
+/***************************************************************************
+ *   Copyright (C) 2003-2006 by Roman Savochenko                           *
  *   rom_as@fromru.com                                                     *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -24,11 +26,12 @@
 #include <string>
 #include <vector>
 
+#include "tcntrnode.h"
 #include "telem.h"
 
 //Element type flags
-#define F_NOVAL   0x1000  //No value mirored
-#define F_KEY     0x2000  //Primary key
+#define FLD_NOVAL   0x10  //No value mirrored
+#define FLD_KEY     0x20  //Primary key
 
 using std::string;
 using std::vector;
@@ -38,32 +41,40 @@ class TConfig;
 class TCfg
 {
     public:
+	//Methods
 	TCfg( TFld &fld, TConfig &owner );
 	~TCfg();
 	
 	const string &name();
 	
-	bool operator==(TCfg & cfg);
-        TCfg &operator=(TCfg & cfg);		
+	bool operator==(TCfg &cfg);
+        TCfg &operator=(TCfg &cfg);		
 	
-	bool  view( ){ return(m_view); }
-	void  view( bool vw ){ m_view = vw; }
+	bool  view( )		{ return m_view; }
+	void  view( bool vw )	{ m_view = vw; }
 	
-	TFld &fld(){ return(*m_fld); }	
+	TFld &fld()		{ return *m_fld; }	
 	
+	//- Universal access -
         string getSEL( );
-	string &getS( );
-	double &getR( );
-	int    &getI( );
-	bool   &getB( );
+	string getS( );
+	double getR( );
+	int    getI( );
+	bool   getB( );
+	//- Direct access -
+        string &getSd( );
+        double &getRd( );
+        int    &getId( );
+        bool   &getBd( );					
 	
 	void setSEL( const string &val );
 	void setS( const string &val );
 	void setR( double val );
 	void setI( int val );
 	void setB( bool val );
-
+	
     private:
+	//Data
     	union 
 	{
 	    string *s_val;
@@ -71,11 +82,12 @@ class TCfg
 	    int    i_val;
 	    bool   b_val;
 	}m_val;	
+	
+	//Attributes
 	bool m_view;
 	
 	TFld     *m_fld;
 	TConfig  &m_owner;	
-	static const char       *o_name;
 };
 
 
@@ -85,8 +97,8 @@ class TConfig: public TValElem
 {
     friend class TCfg;
 
-    /** Public methods: */
     public:
+	//Methods
 	TConfig( TElem *Elements = NULL );
 	~TConfig();
 
@@ -95,26 +107,26 @@ class TConfig: public TValElem
 	void cfgList( vector<string> &list );
 	TCfg &cfg( const string &n_val );
 	
-	void elem(TElem *Elements); 
+	void elem(TElem *Elements, bool first = false); 
 	TElem &elem();
 
     protected:	
-	virtual bool change( TCfg &cfg ){ return(true); }
+	//Methods
+	virtual bool cfgChange( TCfg &cfg )	{ return true; }
 	
-    /** Public methods: */
-    private:
-	// Add elem into TElem
-	void addElem( TElem &el, unsigned id );
-	// Del elem without TElem
-	void delElem( TElem &el, unsigned id );	
+	void cntrMake( XMLNode *fld, const char *req, const char *path, int pos );
+        void cntrCmd( const string &elem, XMLNode *fld, TCntrNode::Command cmd );
 	
-    /**Attributes: */
     private:
+	//Methods
+	void detElem( TElem *el );
+	void addFld( TElem *el, unsigned id );
+	void delFld( TElem *el, unsigned id );	
+	
+	//Attributes
 	vector<TCfg*>	value;
 	TElem   	*m_elem;
         bool     	single;
-	
-	static const char       *o_name;
 };
 
 #endif // TCONFIG_H
