@@ -1,5 +1,7 @@
+
+//OpenSCADA system file: telem.h
 /***************************************************************************
- *   Copyright (C) 2004 by Roman Savochenko                                *
+ *   Copyright (C) 2003-2006 by Roman Savochenko                           *
  *   rom_as@fromru.com                                                     *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -23,48 +25,50 @@
 
 #include <string>
 #include <vector>
-//---- Type ----
-#define T_STRING  0x01
-#define T_DEC     0x02
-#define T_OCT     0x04
-#define T_HEX     0x08
-#define T_REAL    0x10
-#define T_BOOL    0x20
 
-#define T_SELECT  0x40   //Connnect to simple elements
 
-//---- Other flags ----
-#define F_SELF    0x80   //Create self field
-#define F_NWR     0x100  //No writeable
-#define F_PREV    0x200  //Prevent owner for change
+//--------- Flags ------------
+#define FLD_NOFLG   0x00	//No flag
+#define FLD_SELECT  0x01   	//Connnect to simple elements
+#define FLD_SELF    0x02   	//Create self field
+#define FLD_NWR     0x04  	//No writeable
+#define FLD_PREV    0x08  	//Prevent owner for change
 
 using std::string;
 using std::vector;
 
+class XMLNode;
+
 //Internal structures
 class TFld
 {
     public:
+	//Data
+	enum Type { Bool, Dec, Hex, Oct, Real, String };
+	
+	//Methods
 	TFld( );
-	TFld( const string &name, const string &descr, unsigned type,
-	    const string &valLen = "", const string &valDef = "", 
-	    const string &vals = "", const string &nSel = "" );
+	TFld( const char *name, const char *descr, Type itype, unsigned char iflg,
+	    const char *valLen = "", const char *valDef = "", 
+	    const char *vals = "", const char *nSel = "", int w_id = 0 );
 	~TFld();
 	
 	TFld &operator=( TFld &fld );	
 
-	const string &name() { return m_name; }	// Name of element (name column into BD);
-	string &descr()      { return m_descr; }// Description of element;
-	int len()	     { return m_len; }	// field len 
-	int dec()            { return m_dec; }	// field dec (for real)
-	unsigned type()      { return m_type; }	// Type of element (T_STRING, T_DEC, T_OCT, T_HEX, T_REAL, T_SELECT, T_SELECT|T_STRING); 
-	const string &def()  { return m_def; }	// default value;
+	const string &name() 	{ return m_name; }	// Name of element (name column into BD);
+	string &descr()      	{ return m_descr; }	// Description of element;
+	int len()	     	{ return m_len; }	// field len 
+	int dec()            	{ return m_dec; }	// field dec (for real)
+	Type type()		{ return m_type; }	// Type (Dec, Hex, Oct, ...)
+	unsigned char flg()	{ return m_flg; }	// element flags (FLD_SELECT, FLD_SELF ...); 
+	const string &def()  	{ return m_def; }	// default value;
+	int workId()		{ return m_wid; }
 	
 	vector<string> &selValS();
 	vector<int>    &selValI();
 	vector<double> &selValR();
 	vector<bool>   &selValB();
-       	// selectable element's name        
+       	//- selectable element's name -
 	vector<string> &selNm();
 
 	string selVl2Nm( const string &val );
@@ -77,14 +81,19 @@ class TFld
 	double  selNm2VlR( const string &name );	
 	bool    selNm2VlB( const string &name );	
 
+	void 	cntrMake( XMLNode *fld, const char *req, const char *path, int pos );
+	
     private:    
+	//Attributes
 	string          m_name;  
 	string          m_descr; 
 	int		m_len;
 	int		m_dec;
-	unsigned	m_type;  
+	Type		m_type;
+	unsigned char	m_flg;  
 	string          m_def;
-	//vector<string>  m_vals;  
+	int		m_wid;
+	
 	union           
 	{
 	    vector<string> *v_s;
@@ -100,16 +109,17 @@ class TValElem;
 
 class TElem
 {
-    /** Public methods: */
     public:
-	TElem( const string &name );
+	//Methods
+	TElem( const string &name = "" );
 	~TElem();
 	
-	string &elName( ) { return(m_name); }
+	string &elName( )	{ return(m_name); }
 	
 	void fldList( vector<string> &list );
-	unsigned fldSize(){ return(elem.size()); }
+	unsigned fldSize()	{ return(elem.size()); }
 	unsigned fldId( const string &name);	
+	bool fldPresent( const string &name );
 	int fldAdd( TFld *fld, int id = -1 );
 	void fldDel(unsigned int id);
 	TFld &fldAt( unsigned int id );
@@ -117,12 +127,12 @@ class TElem
         void valAtt( TValElem *cnt ); 
         void valDet( TValElem *cnt ); 
 	
-    /**Attributes: */
     private:
-        string             m_name;
-	vector<TFld*>      elem;
-	vector<TValElem*> cont;        //Conteiners
-	static const char  *o_name;
+	//Attributes
+        string       	m_name;
+	vector<TFld*>	elem;
+	vector<TValElem*>	cont;        //Conteiners
+	static const char	*o_name;
 };
 
 //**********************************************************************
@@ -133,15 +143,16 @@ class TValElem
 {
     friend class TElem;
     
-    /** Public methods: */
     public:	
+	//Methods
 	TValElem();
 	virtual ~TValElem();
+	
     protected:
-	// Add element
-	virtual void addElem( TElem &el, unsigned id ) = 0;
-	// Del element
-	virtual void delElem( TElem &el, unsigned id ) = 0;
+	//Methods
+	virtual void detElem( TElem *el ) = 0;
+	virtual void addFld( TElem *el, unsigned id ) = 0;
+	virtual void delFld( TElem *el, unsigned id ) = 0;
 };
 
 
