@@ -1,5 +1,7 @@
+
+//OpenSCADA system file: tprotocols.cpp
 /***************************************************************************
- *   Copyright (C) 2004 by Roman Savochenko                                *
+ *   Copyright (C) 2003-2006 by Roman Savochenko                           *
  *   rom_as@fromru.com                                                     *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -21,37 +23,36 @@
 #include <getopt.h>
 
 #include "tsys.h"
-#include "tkernel.h"
-#include "tmessage.h"
+#include "tmess.h"
 #include "tprotocols.h"
 
 //================================================================
 //=========== TProtocolS =========================================
 //================================================================
-const char *TProtocolS::o_name = "TProtocolS";
-
-TProtocolS::TProtocolS( TKernel *app ) : TGRPModule(app,"Protocol") 
+TProtocolS::TProtocolS( ) : TSubSYS("Protocol","Transport protocols",true) 
 {
-    s_name = "Protocols";
+
+}
+
+TProtocolS::~TProtocolS( )
+{
+
 }
 
 string TProtocolS::optDescr(  )
 {
     return(Mess->I18N(
-    	"======================= The protocol subsystem options ====================\n"
-	"    --PRCModPath=<path>  Set moduls <path>;\n\n"));
+    	"=============== Subsystem \"Transport protocols\" options =================\n\n"));
 }
 
-void TProtocolS::gmdCheckCommandLine( )
+void TProtocolS::subLoad()
 {
-    TGRPModule::gmdCheckCommandLine( );
-    
+    //========== Load parameters from command line ============
     int next_opt;
     char *short_opt="h";
     struct option long_opt[] =
     {
 	{"help"      ,0,NULL,'h'},
-	{"PRCModPath",1,NULL,'m'},
 	{NULL        ,0,NULL,0  }
     };
 
@@ -62,76 +63,62 @@ void TProtocolS::gmdCheckCommandLine( )
 	switch(next_opt)
     	{
 	    case 'h': fprintf(stdout,optDescr().c_str()); break;
-	    case 'm': DirPath = optarg;     break;
 	    case -1 : break;
 	}
     } while(next_opt != -1);
-}    
+    
+    //========== Load parameters from config file =============
 
-void TProtocolS::gmdUpdateOpt()
-{
-    TGRPModule::gmdUpdateOpt();
+    //Load modules
+    TSubSYS::subLoad();
 }
 
 //=========== Control ==========================================
-void TProtocolS::ctrStat_( XMLNode *inf )
+void TProtocolS::cntrCmd_( const string &a_path, XMLNode *opt, TCntrNode::Command cmd )
 {
-    char *dscr = "dscr";
-    
-    TGRPModule::ctrStat_( inf );
-    
-    char *i_help = 
-	"<fld id='g_help' acs='0440' tp='str' cols='90' rows='5'/>";
-    
-    XMLNode *n_add = inf->childGet("id","help")->childAdd();
-    n_add->load(i_help);
-    n_add->attr(dscr,Mess->I18N("Options help"));
+    if( cmd==TCntrNode::Info )
+    {
+	TSubSYS::cntrCmd_( a_path, opt, cmd );       //Call parent
+
+	ctrMkNode("fld",opt,-1,a_path.c_str(),"/help/g_help",Mess->I18N("Options help"),0440,0,0,3,"tp","str","cols","90","rows","5");
+    }
+    else if( cmd==TCntrNode::Get )
+    {
+	if( a_path == "/help/g_help" ) ctrSetS( opt, optDescr() );
+	else TSubSYS::cntrCmd_( a_path, opt, cmd );
+    }
+    else if( cmd==TCntrNode::Set )
+    	TSubSYS::cntrCmd_( a_path, opt, cmd );
 }
 
-void TProtocolS::ctrDinGet_( const string &a_path, XMLNode *opt )
-{
-    if( a_path == "/help/g_help" ) ctrSetS( opt, optDescr() );       
-    else TGRPModule::ctrDinGet_( a_path, opt );
-}
 
 //================================================================
 //=========== TProtocol ==========================================
 //================================================================
-const char *TProtocol::o_name = "TProtocol";
-
-TProtocol::TProtocol() : m_hd(o_name)
+TProtocol::TProtocol()
 {
-
+    m_pr = grpAdd("pr_");
 }
 
 TProtocol::~TProtocol()
 {
-    m_hd.lock();
-    vector<string> list_el;
-    list(list_el);
-    for( unsigned i_ls = 0; i_ls < list_el.size(); i_ls++)
-        close(list_el[i_ls]);
 }
 
 void TProtocol::open( const string &name )
 {
-    if( m_hd.objAvoid(name) ) return;
-    TProtocolIn *t_prt = in_open(name);
-    try { m_hd.objAdd( t_prt, &t_prt->name() ); }
-    catch(TError err) { delete t_prt; throw; }
+    if( chldPresent(m_pr,name) ) return;
+    chldAdd(m_pr,in_open(name)); 
 }
 
 void TProtocol::close( const string &name )
 {
-    delete (TProtocolIn *)m_hd.objDel( name );
+    chldDel(m_pr,name);
 }
 
 //================================================================
 //=========== TProtocolIn ========================================
 //================================================================
-const char *TProtocolIn::o_name = "TProtocolIn";
-
-TProtocolIn::TProtocolIn( const string &name, TProtocol *owner ) : m_name(name), m_owner(owner)
+TProtocolIn::TProtocolIn( const string &name ) : m_name(name)
 {
 
 }
