@@ -1,5 +1,7 @@
+
+//OpenSCADA system file: tprotocols.h
 /***************************************************************************
- *   Copyright (C) 2004 by Roman Savochenko                                *
+ *   Copyright (C) 2003-2006 by Roman Savochenko                           *
  *   rom_as@fromru.com                                                     *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -25,11 +27,7 @@
 
 #include <string>
 
-#include "thd.h"
-#include "tkernel.h"
-#include "tmessage.h"
-#include "tmodule.h"
-#include "tgrpmodule.h"
+#include "tsubsys.h"
 
 using std::string;
 
@@ -38,27 +36,26 @@ using std::string;
 //================================================================
 class TProtocol;
 
-class TProtocolIn
+class TProtocolIn : public TCntrNode
 {
-    /** Public methods: */
     public:
-	TProtocolIn( const string &name, TProtocol *owner );
+	TProtocolIn( const string &name );
 	virtual ~TProtocolIn();
 	
-	string &name(){ return(m_name); }
+	string &name()	{ return(m_name); }
 	
-	// process input messages
+	//- Process input messages -
 	virtual bool mess( const string &request, string &answer, const string &sender )
 	{ answer = ""; }
 	
-	//Owner
-	TProtocol &owner(){ return( *m_owner ); }	
+	TProtocol &owner()	{ return *(TProtocol *)nodePrev(); }
 	
-    private:    
+    private:
+	//Methods
+	string nodeName()	{ return m_name; }
+	
+	//Attributes
 	string            m_name;
-	TProtocol         *m_owner;
-    
-	static const char *o_name;
 };
 
 //================================================================
@@ -66,60 +63,44 @@ class TProtocolIn
 //================================================================
 class TProtocol: public TModule
 {
-    /** Public methods: */
     public:
 	TProtocol( );
-	virtual ~TProtocol();
+	virtual ~TProtocol( );
 
-	// List opened input object protocols
-	void list( vector<string> &list ) { m_hd.objList( list ); }
-	// Open stat
-        bool openStat( const string &name )
-        { return m_hd.objAvoid(name); }
-	// Open input protocol.
+	//- Input protocols -
+	void list( vector<string> &list )	{ chldList(m_pr,list); }
+        bool openStat( const string &name )	{ return chldPresent(m_pr,name); } 
 	void open( const string &name );
-    	// Close input protocol.
 	void close( const string &name );
-	// Go input protocol
 	AutoHD<TProtocolIn> at( const string &name )
-        { AutoHD<TProtocolIn> obj( name, m_hd ); return obj; }
+	{ return chldAt(m_pr,name); }
 	
     private:
+	//Methods
 	virtual TProtocolIn *in_open( const string &name )
-	{throw TError("(%s) Function 'in_open' no support!",o_name); }		
+	{throw TError(nodePath().c_str(),"Function 'in_open' no support!"); }		
 	
-    private:
-	THD               m_hd;
-    
-	static const char *o_name;
+	//Attributes
+	int	m_pr;
 };
 
 //================================================================
 //=========== TProtocolS =========================================
 //================================================================
 
-class TProtocolS : public TGRPModule
+class TProtocolS : public TSubSYS
 {
-    /** Public methods: */
     public:
-	TProtocolS( TKernel *app );
+	TProtocolS( );
+	~TProtocolS( );
     
-	int gmdVer( ) { return(VER_PROT); }
+	int subVer( ) 	{ return(VER_PROT); }
+	void subLoad( );
 
-	void gmdCheckCommandLine( );
-	void gmdUpdateOpt();
-	
 	string optDescr( );	
 
-    /** Private methods: */
     private:
-	//================== Controll functions ========================
-	void ctrStat_( XMLNode *inf );
-	void ctrDinGet_( const string &a_path, XMLNode *opt );
-	
-    /** Private atributes: */
-    private:
-	static const char *o_name;
+	void cntrCmd_( const string &a_path, XMLNode *opt, TCntrNode::Command cmd );
 };
 
 #endif // TPROTOCOLS_H
