@@ -1,5 +1,7 @@
+
+//OpenSCADA system file: tspecials.cpp
 /***************************************************************************
- *   Copyright (C) 2004 by Roman Savochenko                                *
+ *   Copyright (C) 2003-2006 by Roman Savochenko                           *
  *   rom_as@fromru.com                                                     *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -21,37 +23,31 @@
 #include <getopt.h>
 
 #include "tsys.h"
-#include "tkernel.h"
-#include "tmessage.h"
+#include "tmess.h"
 #include "tspecials.h"
 
 //================================================================
 //=========== TSpecialS ==========================================
 //================================================================
-const char *TSpecialS::o_name = "TSpecialS";
-
-TSpecialS::TSpecialS( TKernel *app ) : TGRPModule(app,"Special")  
+TSpecialS::TSpecialS( ) : TSubSYS("Special","Specials",true)
 {
-    s_name = "Specials";
+
 }
 
 string TSpecialS::optDescr( )
 {
     return(Mess->I18N(
-    	"======================= The special subsystem options =====================\n"
-	"    --SPModPath=<path>  Set moduls <path>;\n\n"));
+    	"======================= Subsystem \"Special\" options =====================\n\n"));
 }
 
-void TSpecialS::gmdCheckCommandLine( )
+void TSpecialS::subLoad( )
 {
-    TGRPModule::gmdCheckCommandLine( );
-    
+    //========== Load parameters from command line ============
     int next_opt;
     char *short_opt="h";
     struct option long_opt[] =
     {
 	{"help"     ,0,NULL,'h'},
-	{"SPModPath",1,NULL,'m'},
 	{NULL       ,0,NULL,0  }
     };
 
@@ -62,89 +58,62 @@ void TSpecialS::gmdCheckCommandLine( )
 	switch(next_opt)
 	{
 	    case 'h': fprintf(stdout,optDescr().c_str()); break;
-	    case 'm': DirPath = optarg;     break;
 	    case -1 : break;
 	}
     } while(next_opt != -1);
-}
+    
+    //========== Load parameters from config file =============
 
-void TSpecialS::gmdUpdateOpt()
-{
-    TGRPModule::gmdUpdateOpt();
-}
-
-void TSpecialS::gmdStart( )
-{
-    vector<string> list;
-    gmdList(list);
-    for(unsigned i_sp = 0; i_sp < list.size(); i_sp++)
-	((TSpecial &)gmdAt(list[i_sp]).at()).start();
-}              
-
-void TSpecialS::gmdStop( )
-{
-    vector<string> list;
-    gmdList(list);
-    for(unsigned i_sp = 0; i_sp < list.size(); i_sp++)
-	((TSpecial &)gmdAt(list[i_sp]).at()).stop();
+    //Load modules    
+    TSubSYS::subLoad();
 }
 
 //=========== Control ==========================================
-void TSpecialS::ctrStat_( XMLNode *inf )
+void TSpecialS::cntrCmd_( const string &a_path, XMLNode *opt, TCntrNode::Command cmd )
 {
-    char *dscr = "dscr";
-    
-    TGRPModule::ctrStat_( inf );
-	
-    char *i_help =
-        "<fld id='g_help' acs='0440' tp='str' cols='90' rows='5'/>";
-		    
-    XMLNode *n_add = inf->childGet("id","help")->childAdd();
-    n_add->load(i_help);
-    n_add->attr(dscr,Mess->I18N("Options help"));
+    if( cmd==TCntrNode::Info )
+    {
+	TSubSYS::cntrCmd_( a_path, opt, cmd );       //Call parent
+
+	ctrMkNode("fld",opt,-1,a_path.c_str(),"/help/g_help",Mess->I18N("Options help"),0440,0,0,3,"tp","str","cols","90","rows","5");
+    }
+    else if( cmd==TCntrNode::Get )
+    {
+	if( a_path == "/help/g_help" ) ctrSetS( opt, optDescr() );
+	else TSubSYS::cntrCmd_( a_path, opt, cmd );
+    }
+    else if( cmd==TCntrNode::Set )
+	TSubSYS::cntrCmd_( a_path, opt, cmd );
 }
 
-void TSpecialS::ctrDinGet_( const string &a_path, XMLNode *opt )
-{
-    if( a_path == "/help/g_help" ) ctrSetS( opt, optDescr() );       
-    else TGRPModule::ctrDinGet_( a_path, opt );
-}
+
 //================================================================
 //=========== TSpecial ===========================================
 //================================================================
-const char *TSpecial::o_name = "TSpecial";
-
 TSpecial::TSpecial() : run_st(false)
 {
 
 }
 
 //================== Controll functions ========================
-void TSpecial::ctrStat_( XMLNode *inf )
+void TSpecial::cntrCmd_( const string &a_path, XMLNode *opt, TCntrNode::Command cmd )
 {
-    char *i_cntr = 
-	"<area id='prm'>"
-	" <fld id='st' acs='0664' tp='bool'/>"
-	"</area>";
-    char *dscr = "dscr";
-    
-    TModule::ctrStat_( inf );
-    
-    XMLNode *n_add = inf->childIns(0);
-    n_add->load(i_cntr);
-    n_add->attr(dscr,Mess->I18N("Special"));
-    n_add->childGet(0)->attr(dscr,Mess->I18N("Run"));
-}
+    switch(cmd)
+    {
+	case TCntrNode::Info:
+	    TModule::cntrCmd_( a_path, opt, cmd );
 
-void TSpecial::ctrDinGet_( const string &a_path, XMLNode *opt )
-{
-    if( a_path == "/prm/st" )	ctrSetB( opt, run_st );
-    else TModule::ctrDinGet_( a_path, opt );
-}
-
-void TSpecial::ctrDinSet_( const string &a_path, XMLNode *opt )
-{
-    if( a_path == "/prm/st" )	if( ctrGetB( opt ) ) start(); else stop();
-    else TModule::ctrDinSet_( a_path, opt );
+	    ctrMkNode("area",opt,0,a_path.c_str(),"/prm",Mess->I18N("Special"));
+	    ctrMkNode("area",opt,-1,a_path.c_str(),"/prm/st",Mess->I18N("State"));
+	    ctrMkNode("fld",opt,-1,a_path.c_str(),"/prm/st/st",Mess->I18N("Runing"),0664,0,0,1,"tp","bool");
+	    break;
+	case TCntrNode::Get:
+	    if( a_path == "/prm/st/st" )	ctrSetB( opt, run_st );
+	    else TModule::cntrCmd_( a_path, opt, cmd );
+	    break;
+	case TCntrNode::Set:
+	    if( a_path == "/prm/st/st" )	ctrGetB( opt )?modStart():modStop();
+	    else TModule::cntrCmd_( a_path, opt, cmd );
+    }
 }
 
