@@ -1,5 +1,7 @@
+
+//OpenSCADA system file: tspecials.h
 /***************************************************************************
- *   Copyright (C) 2004 by Roman Savochenko                                *
+ *   Copyright (C) 2003-2006 by Roman Savochenko                           *
  *   rom_as@fromru.com                                                     *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -25,8 +27,7 @@
 
 #include <string>
 
-#include "tmodule.h"
-#include "tgrpmodule.h"
+#include "tsubsys.h"
 
 using std::string;
 
@@ -36,56 +37,33 @@ using std::string;
 
 class TSpecial : public TModule
 {
-    /** Public methods: */
     public:
 	TSpecial( );
 
-	virtual void start( ){ }
-	virtual void stop( ){ }
-	
     protected:
-	//================== Controll functions ========================
-	void ctrStat_( XMLNode *inf );
-	void ctrDinGet_( const string &a_path, XMLNode *opt );
-	void ctrDinSet_( const string &a_path, XMLNode *opt );
+	//Methods
+	void cntrCmd_( const string &a_path, XMLNode *opt, TCntrNode::Command cmd );
 	
-    protected:
+	//Attributes
 	bool  run_st; 
-	
-    /** Private atributes: */
-    private:
-	static const char *o_name;
 };
 
 //================================================================
 //=========== TSpecialS ==========================================
 //================================================================
 
-class TSpecialS : public TGRPModule
+class TSpecialS : public TSubSYS
 {
-    /** Public methods: */
     public:
-	TSpecialS( TKernel *app );
+	TSpecialS( );
 
-	int gmdVer( ) { return(VER_SPC); }
+	int subVer( ) { return(VER_SPC); }
+	void subLoad( );
 
-	void gmdStart( );
-	void gmdStop( );
-
-	void gmdCheckCommandLine( );
-	void gmdUpdateOpt();
-	
 	string optDescr( );
 	
-    /** Private methods: */
     private:
-	//================== Controll functions ========================
-	void ctrStat_( XMLNode *inf );
-	void ctrDinGet_( const string &a_path, XMLNode *opt );
-	
-    /** Private atributes: */
-    private:
-	static const char *o_name;
+	void cntrCmd_( const string &a_path, XMLNode *opt, TCntrNode::Command cmd );
 };
 
 #endif // TSPECIALS_H
