@@ -1,5 +1,7 @@
+
+//OpenSCADA system file: ttiparam.h
 /***************************************************************************
- *   Copyright (C) 2004 by Roman Savochenko                                *
+ *   Copyright (C) 2003-2006 by Roman Savochenko                           *
  *   rom_as@fromru.com                                                     *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -33,12 +35,13 @@ using std::vector;
 class TTipParam : public TElem
 {
     public:
-	TTipParam( const string &name, const string &descr, const string &bd );
+	TTipParam( const char *id, const char *name, const char *db );
 	~TTipParam(  );
 	
 	string name()  { return(_name); }
 	string lName() { return(_descr); }
 	string BD()    { return(_bd); }
+	
     private:
 	string _name;
 	string _descr;
