@@ -1,5 +1,7 @@
+
+//OpenSCADA system file: tuis.cpp
 /***************************************************************************
- *   Copyright (C) 2004 by Roman Savochenko                                *
+ *   Copyright (C) 2003-2006 by Roman Savochenko                           *
  *   rom_as@fromru.com                                                     *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -18,40 +20,37 @@
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
 
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <fcntl.h>
 #include <getopt.h>
 
 #include "tsys.h"
-#include "tkernel.h"
-#include "tmessage.h"
+#include "tmess.h"
 #include "tuis.h"
 
 //================================================================
 //================== TUIS ========================================
 //================================================================
-const char *TUIS::o_name = "TUIS";
-	
-TUIS::TUIS( TKernel *app ) : TGRPModule(app,"UI") 
+TUIS::TUIS( ) : TSubSYS("UI","User interfaces",true)
 {
-    s_name = "User interfaces"; 
+
 }
 
 string TUIS::optDescr( )
 {
     return(Mess->I18N(
-    	"===================== The user interface subsystem options ================\n"
-	"    --GUIModPath=<path>  Set moduls <path>;\n\n"));
+    	"===================== Subsystem \"User interfaces\" options ===============\n\n"));
 }
 
-void TUIS::gmdCheckCommandLine( )
+void TUIS::subLoad( )
 {
-    TGRPModule::gmdCheckCommandLine( );
-
+    //========== Load parameters from command line ============
     int next_opt;
     char *short_opt="h";
     struct option long_opt[] =
     {
 	{"help"      ,0,NULL,'h'},
-	{"GUIModPath",1,NULL,'m'},
 	{NULL        ,0,NULL,0  }
     };
 
@@ -62,90 +61,91 @@ void TUIS::gmdCheckCommandLine( )
 	switch(next_opt)
 	{
 	    case 'h': fprintf(stdout,optDescr().c_str()); break;
-	    case 'm': DirPath = optarg;     break;
 	    case -1 : break;
 	}
     } while(next_opt != -1);
-}
-
-void TUIS::gmdUpdateOpt()
-{
-    TGRPModule::gmdUpdateOpt();
-
-}
-
-void TUIS::gmdStart( )
-{
-    vector<string> list;
-    gmdList(list);
-    for(unsigned i_sp = 0; i_sp < list.size(); i_sp++)
-	((TUI &)gmdAt(list[i_sp]).at()).start();
-}
-
-void TUIS::gmdStop( )
-{
-    vector<string> list;
-    gmdList(list);
-    for(unsigned i_sp = 0; i_sp < list.size(); i_sp++)
-	((TUI &)gmdAt(list[i_sp]).at()).stop();
-}
-
-//=========== Control ==========================================
-void TUIS::ctrStat_( XMLNode *inf )
-{
-    char *dscr = "dscr";
-    TGRPModule::ctrStat_( inf );
     
-    //Insert to Help
-    char *i_help = "<fld id='g_help' acs='0440' tp='str' cols='90' rows='5'/>";
-    
-    XMLNode *n_add = inf->childGet("id","help")->childAdd();    
-    n_add->load(i_help);
-    n_add->attr(dscr,Mess->I18N("Options help"));
+    //========== Load parameters from config file =============
+
+    //Load modules
+    TSubSYS::subLoad();
 }
 
-void TUIS::ctrDinGet_( const string &a_path, XMLNode *opt )
+bool TUIS::presentIco(const string &inm, string *tp)
 {
-    if( a_path == "/help/g_help" ) ctrSetS( opt, optDescr() );       
-    else TGRPModule::ctrDinGet_( a_path, opt );
+    int hd = open((string("./icons/")+inm+".png").c_str(),O_RDONLY);
+    if( hd != -1 )
+    {
+	if( tp ) *tp = "png";
+	close(hd);
+	return true;
+    }
+    return false;    
 }
+
+string TUIS::getIco(const string &inm, string *tp )
+{
+    int len;
+    char buf[STR_BUF_LEN];
+    string rez;
+    
+    int hd = open((string("./icons/")+inm+".png").c_str(),O_RDONLY);
+    if( hd != -1 )
+    {
+        if( tp ) *tp = "png";
+        while( len = read(hd,buf,sizeof(buf)) )
+    	    rez.append(buf,len);
+        close(hd);
+    }
+    
+    return rez;
+}
+
+void TUIS::cntrCmd_( const string &a_path, XMLNode *opt, TCntrNode::Command cmd )
+{
+    if( cmd==TCntrNode::Info )
+    {
+	TSubSYS::cntrCmd_( a_path, opt, cmd );       //Call parent
+
+	ctrMkNode("fld",opt,-1,a_path.c_str(),"/help/g_help",Mess->I18N("Options help"),0440,0,0,3,"tp","str","cols","90","rows","5");
+    }
+    else if( cmd==TCntrNode::Get )
+    {
+    	if( a_path == "/help/g_help" ) ctrSetS( opt, optDescr() );
+	else TSubSYS::cntrCmd_( a_path, opt, cmd );
+    }
+    else if( cmd==TCntrNode::Set )
+	TSubSYS::cntrCmd_( a_path, opt, cmd );
+}
+
+
 //================================================================
 //================== TUI =========================================
 //================================================================
-const char *TUI::o_name = "TUI";
-
 TUI::TUI() : run_st(false)
 {
 
 }
     
-//================== Controll functions ========================
-void TUI::ctrStat_( XMLNode *inf )
+void TUI::cntrCmd_( const string &a_path, XMLNode *opt, TCntrNode::Command cmd )
 {
-    char *dscr = "dscr";
-    
-    TModule::ctrStat_( inf );
-    
-    char *i_cntr = 
-	"<area id='a_prm'>"
-	" <fld id='r_st' acs='0664' tp='bool'/>"
-	"</area>";
-    
-    XMLNode *n_add = inf->childIns(0);
-    n_add->load(i_cntr);
-    n_add->attr(dscr,Mess->I18N("User interface"));
-    n_add->childGet(0)->attr(dscr,Mess->I18N("Runing"));
-}
-
-void TUI::ctrDinGet_( const string &a_path, XMLNode *opt )
-{
-    if( a_path == "/a_prm/r_st" ) ctrSetB( opt, run_st );
-    else TModule::ctrDinGet_( a_path, opt );
-}
-
-void TUI::ctrDinSet_( const string &a_path, XMLNode *opt )
-{
-    if( a_path == "/a_prm/r_st" ) if( ctrGetB( opt ) ) start(); else stop();
-    else TModule::ctrDinSet_( a_path, opt );
+    if( cmd==TCntrNode::Info )
+    {
+	TModule::cntrCmd_( a_path, opt, cmd );       //Call parent
+	
+	ctrMkNode("area",opt,0,a_path.c_str(),"/prm",Mess->I18N("User interface"));
+	ctrMkNode("area",opt,-1,a_path.c_str(),"/prm/st",Mess->I18N("State"));
+	ctrMkNode("fld",opt,-1,a_path.c_str(),"/prm/st/r_st",Mess->I18N("Runing"),0664,0,0,1,"tp","bool");
+    }
+    else if( cmd==TCntrNode::Get )
+    {
+	if( a_path == "/prm/st/r_st" )	ctrSetB( opt, run_st );
+	else TModule::cntrCmd_( a_path, opt, cmd );
+    }
+    else if( cmd==TCntrNode::Set )
+    {
+	if( a_path == "/prm/st/r_st" )	ctrGetB( opt )?modStart():modStop();
+	else TModule::cntrCmd_( a_path, opt, cmd );
+    }
 }
 
