@@ -1,5 +1,7 @@
+
+//OpenSCADA system file: tuis.h
 /***************************************************************************
- *   Copyright (C) 2004 by Roman Savochenko                                *
+ *   Copyright (C) 2003-2006 by Roman Savochenko                           *
  *   rom_as@fromru.com                                                     *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -26,8 +28,7 @@
 
 #include <string>
 
-#include "tmodule.h"
-#include "tgrpmodule.h"
+#include "tsubsys.h"
 
 using std::string;
 //================================================================
@@ -36,56 +37,37 @@ using std::string;
 
 class TUI : public TModule
 {
-    /** Public methods: */
     public:
      	TUI( );
 
-	virtual void start( ){ }
-	virtual void stop( ){ }	
-	
     protected:
-	//================== Controll functions ========================
-	void ctrStat_( XMLNode *inf );
-	void ctrDinGet_( const string &a_path, XMLNode *opt );
-	void ctrDinSet_( const string &a_path, XMLNode *opt );
+	//Methods
+	void cntrCmd_( const string &a_path, XMLNode *opt, TCntrNode::Command cmd );
 	
-    protected:
+	//Attributes
 	bool  run_st; 
-	
-    /** Private atributes: */
-    private:
-	static const char *o_name;
 };
 
 //================================================================
 //================== TUIS ========================================
 //================================================================
 
-class TUIS : public TGRPModule
+class TUIS : public TSubSYS
 {
-    /** Public methods: */
     public:
-	TUIS( TKernel *app );
+	TUIS( );
 
-	int gmdVer( ) { return(VER_UI); }
-
-	void gmdStart( );
-	void gmdStop( );
-
-	void gmdCheckCommandLine( );
-	void gmdUpdateOpt();
+	int subVer( ) { return(VER_UI); }
+	void subLoad( );
 	
+	//- All system UI functions -
+	static bool presentIco(const string &inm, string *tp = NULL);
+	static string getIco(const string &inm, string *tp = NULL);
+
 	string optDescr( );
 
-    /** Private methods: */
     private:
-	//================== Controll functions ========================
-	void ctrStat_( XMLNode *inf );
-	void ctrDinGet_( const string &a_path, XMLNode *opt );
-	
-    /** Private atributes: */
-    private:
-	static const char *o_name;
+	void cntrCmd_( const string &a_path, XMLNode *opt, TCntrNode::Command cmd );
 };
 
 #endif // TUIS_H
