@@ -1,5 +1,7 @@
+
+//OpenSCADA system file: xml.cpp
 /***************************************************************************
- *   Copyright (C) 2004 by Roman Savochenko                                *
+ *   Copyright (C) 2003-2006 by Roman Savochenko                           *
  *   rom_as@fromru.com                                                     *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -23,6 +25,30 @@
 
 const char *XMLNode::o_name = "XMLNode";
 
+XMLNode &XMLNode::operator=(XMLNode &prm)    
+{
+    //Delete self children and atributes
+    n_attr.clear();
+    v_attr.clear();
+    for( int i_ch = 0; i_ch < m_children.size(); i_ch++ )
+	delete m_children[i_ch];
+    m_children.clear();    	
+    
+    //Copy params (name,text and atributes)
+    name( prm.name() );
+    text( prm.text() );
+    vector<string> ls;
+    prm.attrList(ls);
+    for( int i_atr = 0; i_atr < ls.size(); i_atr++)
+	attr(ls[i_atr],prm.attr(ls[i_atr]));
+
+    //Recursive copy children
+    for( int i_ch = 0; i_ch < prm.childSize(); i_ch++ )
+	*childAdd() = *prm.childGet(i_ch);
+    
+    return *this;
+}
+
 void XMLNode::childAdd( XMLNode * n )
 {
     if( n )  m_children.push_back( n );
@@ -38,7 +64,7 @@ XMLNode* XMLNode::childAdd( const string &name )
 
 void XMLNode::childDel( const unsigned id )
 {
-    if( id >= childSize() ) throw TError("(%s) Child %d no avoid!",o_name,id);
+    if( id >= childSize() ) throw TError(o_name,"Child %d no present.",id);
     delete m_children[id];
     m_children.erase( m_children.begin()+id );
 }
@@ -70,7 +96,7 @@ XMLNode* XMLNode::childIns( unsigned id, const string &name )
 
 XMLNode* XMLNode::childGet( const int index ) const
 {
-    if( index >= childSize() ) throw TError("(%s) Child %d no avoid!",o_name,index);
+    if( index >= childSize() ) throw TError(o_name,"Child %d no present.",index);
     return( m_children[index] );
 }
 
@@ -80,15 +106,16 @@ XMLNode* XMLNode::childGet( const string &name, const int numb ) const
 	if( childGet(i_ch)->name() == name )
 	    if( i_n++ == numb ) return( childGet(i_ch) );
 	    
-    throw TError("(%s) Child %s:%d no found!", o_name, name.c_str(), numb);
+    throw TError(o_name,"Child %s:%d no found!",name.c_str(),numb);
 }
 
-XMLNode* XMLNode::childGet( const string &attr, const string &val ) const
+XMLNode* XMLNode::childGet( const string &attr, const string &val, bool noex ) const
 {
     for( unsigned i_f = 0; i_f < childSize(); i_f++)
 	if( childGet(i_f)->attr(attr) == val ) return( childGet(i_f) );
 	
-    throw TError("(%s) Child with attribut %s=%s no avoid!",o_name,attr.c_str(),val.c_str());
+    if( noex ) return NULL;
+    else throw TError(o_name,"Child with attribut %s=%s no present.",attr.c_str(),val.c_str());
 }
 
 void XMLNode::attrList( vector<string> & list ) const
@@ -106,23 +133,27 @@ string XMLNode::attr( const string &name ) const
     return("");
 }
 
-void XMLNode::attr( const string &name, const string &val, const bool add )
+XMLNode* XMLNode::attr_( const char *name, const char *val )
+{
+    return attr( name, val );
+}
+
+XMLNode* XMLNode::attr( const string &name, const string &val )
 {
     for(unsigned i_opt = 0; i_opt < n_attr.size(); i_opt++)
 	if(n_attr[i_opt] == name)
 	{
 	    v_attr[i_opt] = val;  
-	    return;
+	    return this;
 	}
 
-    if(add)
-    {
-	n_attr.push_back(name);
-	v_attr.push_back(val);
-    }
+    n_attr.push_back(name);
+    v_attr.push_back(val);
+    
+    return this;
 }
 
-void XMLNode::clean()
+void XMLNode::clear()
 {
     n_attr.clear();	
     v_attr.clear();	
@@ -134,22 +165,22 @@ void XMLNode::clean()
     m_current_node = 0;
 }
 
-string XMLNode::save( bool humen ) const
+string XMLNode::save( unsigned char flg )
 {
-    string xml = string("<") + encode( name() );
+    string xml = ((flg&XML_BR_OPEN_PREV)?"\n<":"<") + encode( name() );
 
     for(unsigned i_atr = 0; i_atr < n_attr.size(); i_atr++)
 	xml = xml + " " + n_attr[i_atr] + "=\"" + v_attr[i_atr] + "\"";
     
-    xml = xml + ((humen)?">\n":">") + encode( text() ) + ((humen)?"\n":"");
+    xml = xml + ((flg&XML_BR_OPEN_PAST)?">\n":">") + encode( text() ) + ((flg&XML_BR_TEXT_PAST)?"\n":"");
 
     for( int child_index = 0; child_index < childSize(); child_index++ )
     {
 	XMLNode *child = childGet( child_index );
-	if( child )  xml += child->save(humen);
+	if( child )  xml += child->save(flg);
     }
 
-    xml+= string("</") + encode( name() ) + ((humen)?">\n":">");
+    xml+= string("</") + encode( name() ) + ((flg&XML_BR_CLOSE_PAST)?">\n":">");
 
     return xml;
 }
@@ -174,17 +205,17 @@ string XMLNode::encode( const string &s ) const
 
 void XMLNode::load( const string &s )
 {
-    clean();
+    clear();
 
-    XML_Parser p = XML_ParserCreate ( NULL );
-    if( ! p ) throw TError( "(%s) Couldn't allocate memory for parser.",o_name );
+    XML_Parser p = XML_ParserCreate ( "UTF-8" );
+    if( ! p ) throw TError(o_name,"Couldn't allocate memory for parser.");
 
     XML_SetElementHandler( p, start_element, end_element );
     XML_SetCharacterDataHandler( p, characters );
     XML_SetUserData ( p, this );    
 
     if( !XML_Parse( p, s.c_str(), s.size(), true ) )
-        throw TError( "(%s) Parse error at line %d --- %s", o_name, XML_GetCurrentLineNumber(p), XML_ErrorString(XML_GetErrorCode(p)) );
+        throw TError(o_name,"Parse error at line %d --- %s", XML_GetCurrentLineNumber(p), XML_ErrorString(XML_GetErrorCode(p)) );
     XML_ParserFree( p );    
     if( m_root )
     {
