@@ -1,5 +1,7 @@
+
+//OpenSCADA system file: xml.h
 /***************************************************************************
- *   Copyright (C) 2004 by Roman Savochenko                                *
+ *   Copyright (C) 2003-2006 by Roman Savochenko                           *
  *   rom_as@fromru.com                                                     *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -21,6 +23,11 @@
 #ifndef XML_H
 #define XML_H
 
+#define XML_BR_OPEN_PREV  0x01	//Break preview open tag
+#define XML_BR_OPEN_PAST  0x02	//Break past open tag
+#define XML_BR_CLOSE_PAST 0x04	//Break past close tag
+#define XML_BR_TEXT_PAST  0x08	//Break past text
+
 #include <string>
 #include <vector>
 
@@ -34,23 +41,26 @@ class XMLNode
     public:
     	XMLNode( const string &name = "" ) : 
     	    m_root(NULL), m_current_node(NULL), m_name ( name ), m_text("") {  }
-	~XMLNode() { clean(); }
+	~XMLNode() { clear(); }
 
-	string 	name() const { return m_name; }
+	XMLNode &operator=(XMLNode &prm);
+
+	string 	name() const 		{ return m_name; }
         void 	name( const string &s ) { m_name = s; }		
 
-	string	text() const { return m_text; }
+	string	text() const 		{ return m_text; }
         void 	text( const string &s ) { m_text = s; }		
 	
 	void	attrList( vector<string> & list ) const;
         string 	attr( const string &name ) const;
-        void 	attr( const string &name, const string &val, const bool add = true );		
+        XMLNode* attr( const string &name, const string &val );
+	XMLNode* attr_( const char *name, const char *val );
 
-	void 	load( const string & );	
-	string 	save( bool humen = false ) const;
-	void 	clean();
+	void 	load( const string & );
+	string 	save( unsigned char flgs = 0 );
+	void 	clear();
 
-	int 	childSize() const { return m_children.size(); }
+	int 	childSize() const 	{ return m_children.size(); }
 	void 	childAdd( XMLNode * );
 	XMLNode* childAdd( const string &name = "" );
 	int 	childIns( unsigned id, XMLNode * );
@@ -59,7 +69,7 @@ class XMLNode
 	void 	childClean( const string &name = "" );
 	XMLNode* childGet( const int ) const;
 	XMLNode* childGet( const string &name, const int numb = 0 ) const;	
-	XMLNode* childGet( const string &attr, const string &name ) const;	
+	XMLNode* childGet( const string &attr, const string &name, bool noex = false ) const;	
 
     private:
 	string encode ( const string &s ) const;
